﻿using Plane.Collections;
using Plane.Copters;
using Plane.FormationCreator.Formation;
using Plane.Logging;
using Microsoft.Maps.MapControl.WPF;
using Microsoft.Practices.ServiceLocation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Shapes;

namespace Plane.FormationCreator.Views
{
    public partial class MapView
    {
        private class CopterDrawing
        {
            public CopterDrawing(ICopter copter, Map map)
            {
                this.Copter = copter;
                _map = map;
                if (copter is Copter)
                {
                    _brush = ((Copter)copter).Brush;
                    _color = ((SolidColorBrush)_brush).Color;
                }
                else
                {
                    _color = _colors[(_colorIndex++ % _colors.Length)];
                    _brush = new SolidColorBrush(_color);
                }

                _flightTaskManager.TasksCleared += (sender, e) =>
                {
                    for (int i = this.Route.Locations.Count - 1; i >= 1; i--)
                    {
                        this.Route.Locations.RemoveAt(i);
                    }
                    foreach (var wp in this.Waypoints)
                    {
                        _map.Children.Remove(wp);
                    }
                    this.Waypoints.Clear();
                };

                //_map.ViewChangeOnFrame += new EventHandler<MapEventArgs>(Map_ViewChanged);
                

                selectMarkup.Width = 7;
                selectMarkup.Height = 7;
                selectMarkup.HorizontalAlignment = HorizontalAlignment.Left;
                selectMarkup.VerticalAlignment = VerticalAlignment.Top;
                selectMarkup.Fill = new SolidColorBrush(Colors.Black);
            }

            Rectangle selectMarkup = new Rectangle();

            static DropShadowEffect _selectedEffect = new DropShadowEffect();

            static SolidColorBrush _selectedTaskStroke = new SolidColorBrush(Colors.White);

            static Color[] _colors = new Color[]
            {
                Color.FromArgb(180, 255, 0, 0),
                Color.FromArgb(180, 235, 97, 0),
                Color.FromArgb(180, 255, 255, 0),
                Color.FromArgb(180, 0, 255, 0),
                Color.FromArgb(180, 0, 198, 255),
                Color.FromArgb(180, 0, 122, 204),
                Color.FromArgb(180, 174, 0, 255)
            };

            static int _colorIndex = 0;

            const double COPTER_RADIUS = 12;
            const double WAYPOINT_RADIUS = 6;
           

            public ICopter Copter { get; set; }
            public Grid DotContainer { get; set; }
            public Polygon Dot { get; set; }

            public TextBlock CopterText { get; set; }

            public MapPolyline Track { get; set; }
            public Location LastLocation { get; set; }

            public MapPolyline Route { get; set; }
            public List<ShapesContainer> Waypoints { get; set; } = new List<ShapesContainer>();

            Map _map;
            Color _color;
            Brush _brush;

            ILogger _logger = ServiceLocator.Current.GetInstance<ILogger>();
            CopterManager _copterManager = ServiceLocator.Current.GetInstance<CopterManager>();
            FlightTaskManager _flightTaskManager = ServiceLocator.Current.GetInstance<FlightTaskManager>();


            private void Map_ViewChanged<MapEventArgs>(object sender, MapEventArgs e)
            {
                for (int index = 1; index < _flightTaskManager.Tasks.Count; index++)
                {
                    var info = _flightTaskManager.Tasks[index].SingleCopterInfos.Find(i => i.Copter == this.Copter); ;
                    ShapesContainer wpContainer = Waypoints[index - 1];  //起飞任务没有Waypoint;
                    Location location = new Location(info.TargetLat, info.TargetLng);
                    Point wpPos = _map.LocationToViewportPoint(location);
                    wpPos.X -= WAYPOINT_RADIUS;
                    wpPos.Y -= WAYPOINT_RADIUS;
                    MapLayer.SetPosition(wpContainer, _map.ViewportPointToLocation(wpPos));
                }

            }

            public void AddOrMoveCopter()
            {
                try
                {
                    var location = new Location(Copter.Latitude, Copter.Longitude, Copter.Altitude);

                    if (this.Dot != null )
                    {
                        if (Copter is PLCopter && !Copter.IsGpsAccurate)
                            this.Dot.Fill = new SolidColorBrush(Color.FromRgb(0, 0, 0));
                        else
                            this.Dot.Fill = _brush;
                    }

                    
                        
                    Point center = _map.LocationToViewportPoint(location);
                    bool locationUpdated = true;
                    SolidColorBrush blackBrush = new SolidColorBrush();
                    if (this.Dot == null)
                    {
                        Track = new MapPolyline();
                        Track.Stroke = _brush;
                        Track.StrokeThickness = 3.0;
                        Track.Locations = new LocationCollection();
                       
                        //实时飞行航线
                        // _map.Children.Add(Track);

                        Dot = new Polygon();

                        Dot.Points.Add(new Point(0, -COPTER_RADIUS));
                        Dot.Points.Add(new Point(-COPTER_RADIUS * 2 / 3, COPTER_RADIUS));
                        Dot.Points.Add(new Point(COPTER_RADIUS * 2 / 3, COPTER_RADIUS));

                       
                        Dot.Fill = _brush;  // new SolidColorBrush(Color.FromArgb(200, 0, 122, 204));                           
                        Dot.StrokeThickness  = 1;
                        Dot.Width = COPTER_RADIUS * 2;
                        Dot.Height = COPTER_RADIUS * 2;
                        ToolTip tt = new ToolTip();
                        tt.Content = $"Name: {Copter.Name}, Location: {location}";
                        Dot.ToolTip = tt;

                        DotContainer = new Grid { Tag = COPTER_TAG };
                        DotContainer.Children.Add(Dot);

                        //飞机里面的编号


                        CopterText = new TextBlock
                        {
                            Text = Copter.Name,
                            Foreground = new SolidColorBrush(Colors.White),
                            Margin = new Thickness(-COPTER_RADIUS * 2, -COPTER_RADIUS * 1.5, 0, 0),
                            HorizontalAlignment = HorizontalAlignment.Center,
                            VerticalAlignment = VerticalAlignment.Center
                        };


                        DotContainer.Children.Add(CopterText);

                        _map.Children.Add(DotContainer);
                        MapLayer.SetZIndex(DotContainer, 100);

                        this.Route = new MapPolyline { Locations = new LocationCollection { location } };
                        this.Route.Stroke = _brush;
                        this.Route.StrokeThickness = 2.0;
                        //显示计划航线

                        //     _map.Children.Add(this.Route);



                        this.LastLocation = location;

                        RegisterEventHandlersForDraggingCopter(DotContainer);
                    }
                    else
                    {

                        
                        if ((bool)_copterManager.CopterStatus[_copterManager.Copters.IndexOf(Copter)])
                        {
                            CopterText.Foreground = new SolidColorBrush(Colors.Red);
                        }
                        else {

                            CopterText.Foreground = new SolidColorBrush(Colors.White);
                        }



                        if (LastLocation != null && location.CalcDistance(LastLocation) < 0.1)
                        {
                            locationUpdated = false;
                        }
                        if (locationUpdated)
                        {
                            //dot.RenderTransform = new RotateTransform(GeographyUtils.CalcDirection2D(lastLocation.Latitude, lastLocation.Longitude, location.Latitude, location.Longitude).RadiansToDegrees());
                            (Dot.ToolTip as ToolTip).Content = $"Name: {Copter.Name}, Location: {location}";
                        }
                    }

                    if (Copter.GetShowLEDAsync())
                        blackBrush.Color = Colors.White;
                    else
                         blackBrush.Color = Colors.Transparent;
                   Dot.Stroke = blackBrush;

                    var trans = Dot.RenderTransform as RotateTransform;
                    if (trans == null)
                    {
                        Dot.RenderTransform = new RotateTransform(Copter.Heading);
                    }
                    else
                    {
                        if (trans.Angle != Copter.Heading)
                        {
                            trans.Angle = Copter.Heading;
                        }
                    }

                    if (locationUpdated)
                    {
                        Track.Locations.Add(location);
                        MapLayer.SetPosition(Dot.Parent, location);
                        this.LastLocation = location;
                    }
                }
                catch (Exception ex)
                {
                    _logger.Log(ex);
                }
            }

            public void AddWaypoint(Location location, FlightTaskType type)
            {
                // Add waypoint.

               
                ShapesContainer shapesContainer = new ShapesContainer(_brush);
                shapesContainer.Tag = WAYPOINT_TAG;

                Waypoints.Add(shapesContainer);
                _map.Children.Add(shapesContainer);
                MapLayer.SetZIndex(shapesContainer, 100);
                var wpPos = _map.LocationToViewportPoint(location);
                //wpPos.X -= WAYPOINT_RADIUS;
                //wpPos.Y -= WAYPOINT_RADIUS;
                MapLayer.SetPosition(shapesContainer, _map.ViewportPointToLocation(wpPos));

                SetEffect(_copterManager.SelectedCopters.Contains(Copter));

                // Register event handlers.
                RegisterEventHandlersForDraggingWaypoint(shapesContainer, taskIndex: Waypoints.Count);

                // Register event handlers for task info.
                RegisterEventHandlersForTaskInfo(shapesContainer, taskIndex: Waypoints.Count);
            }

            private void RegisterEventHandlersForDraggingCopter(Grid copterElement)
            {
                var dragMoving = false;
                double offsetX = 0;
                double offsetY = 0;
                copterElement.MouseLeftButtonDown += (sender, e) =>
                {
                    _copterManager.Select(this.Copter);
                    if (Copter is FakeCopter)
                    {
                        var posInObject = e.GetPosition(copterElement);
                        offsetX = posInObject.X;
                        offsetY = posInObject.Y;

                        dragMoving = true;
                    }
                };
                _map.MouseMove += (sender, e) =>
                {
                    if (dragMoving)
                    {
                        var pos = e.GetPosition(_map);
                        pos.X -= offsetX;
                        pos.Y -= offsetY;
                        MapLayer.SetPosition(copterElement, _map.ViewportPointToLocation(pos));

                        var center = _map.ViewportPointToLocation(pos);

//                         var routePoint = this.Route.Locations[0];
//                         routePoint.Latitude = center.Latitude;
//                         routePoint.Longitude = center.Longitude;

                        var fc = Copter as FakeCopter;
                        fc.SetProperties(
                            latitude: center.Latitude,
                            longitude: center.Longitude
                        );
                    }
                };
                _map.MouseLeftButtonUp += (sender, e) =>
                {
                    if (dragMoving) dragMoving = false;
                };
            }

            private Dictionary<ICopter, Point> selectWayOriginPoint = new Dictionary<ICopter, Point>();
            private Dictionary<object, bool> _dictDraggingWp = new Dictionary<object, bool>();
            private void RegisterEventHandlersForDraggingWaypoint(Grid wp, int taskIndex)
            {
                _dictDraggingWp[wp] = false;
                double originX = 0;
                double originY = 0;
                double wpOffsetX = 0;
                double wpOffsetY = 0;
                wp.MouseLeftButtonDown += (sender, e) =>
                {
                    //判断点击的航点   
                    //非SelectedTask中的航点 或者 当前SelectedTask中未选择的航点  单选拖动
                    //否则多选拖动
                    if (_flightTaskManager.SelectedTaskIndex != taskIndex || !_copterManager.SelectedCopters.Contains(this.Copter))
                    {
                        //_copterManager.Select(null);
                        _copterManager.Select(this.Copter);
                        _flightTaskManager.Select(taskIndex, this.Copter);
                    }
                    
                    var originPoint = e.GetPosition(_map);
                    originX = originPoint.X;
                    originY = originPoint.Y;

                    var posInObject = e.GetPosition(wp);
                    wpOffsetX = posInObject.X;
                    wpOffsetY = posInObject.Y;

                    _dictDraggingWp[wp] = true;

                    selectWayOriginPoint.Clear();
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.SelectedCopters.Contains(info.Copter))
                        {
                            Location originLocation = new Location(info.TargetLat, info.TargetLng);
                            selectWayOriginPoint[info.Copter] = _map.LocationToViewportPoint(originLocation);
                        }
                    }
                };
                wp.MouseRightButtonDown += (sender, e) =>
                {
                    _flightTaskManager.RightSelect(taskIndex, this.Copter);
                };
                _map.MouseMove += (sender, e) =>
                {
                    if (_dictDraggingWp[wp])
                    {
                        var eventPos = e.GetPosition(_map);
                        var offsetX = eventPos.X - originX;
                        var offsetY = eventPos.Y - originY;

                        foreach (KeyValuePair<ICopter, Point> kv in selectWayOriginPoint)
                        {
                            Point curPoint = new Point(kv.Value.X + offsetX, kv.Value.Y + offsetY);
                            var curLoc = _map.ViewportPointToLocation(curPoint);
                           
                            var modifyingSingleCopterInfo = _flightTaskManager.SelectedTask.SingleCopterInfos.Find(i => i.Copter == kv.Key);
                            //modifyingSingleCopterInfo.TargetLat = routePoint.Latitude = curLoc.Latitude;
                            //modifyingSingleCopterInfo.TargetLng = routePoint.Longitude = curLoc.Longitude;
                            modifyingSingleCopterInfo.TargetLat =  curLoc.Latitude;
                            modifyingSingleCopterInfo.TargetLng =  curLoc.Longitude;

                        }

                        var routePoint = this.Route.Locations[1];
                        var leftTopPos = new Point(eventPos.X - wpOffsetX, eventPos.Y - wpOffsetY);
                        var newRoutePoint = new Point(leftTopPos.X + WAYPOINT_RADIUS, leftTopPos.Y + WAYPOINT_RADIUS);
                        routePoint.Latitude = _map.ViewportPointToLocation(leftTopPos).Latitude;
                        routePoint.Longitude = _map.ViewportPointToLocation(leftTopPos).Longitude;
                        MapLayer.SetPosition(wp, _map.ViewportPointToLocation(leftTopPos));
                    }
                };
                _map.MouseLeftButtonUp += (sender, e) =>
                {
                    if (_dictDraggingWp[wp]) _dictDraggingWp[wp] = false;
                };
            }

            private void RegisterEventHandlersForTaskInfo(Grid wp, int taskIndex)
            {
                var info = _flightTaskManager.Tasks[taskIndex].SingleCopterInfos.FirstOrDefault(i => i.Copter == this.Copter);
                if (info == null) return;
                info.PropertyChanged += (sender, e) =>
                {
                    switch (e.PropertyName)
                    {
                        case nameof(FlightTaskSingleCopterInfo.TargetLat):
                        case nameof(FlightTaskSingleCopterInfo.TargetLng):
                        case nameof(FlightTaskSingleCopterInfo.TargetAlt):
                            if (!_dictDraggingWp.GetValue(wp, false))
                            {
                                var centerLocation = new Location(info.TargetLat, info.TargetLng, info.TargetAlt);
                                var centerPos = _map.LocationToViewportPoint(centerLocation);
                                var leftTopPos = new Point(centerPos.X, centerPos.Y);
                                MapLayer.SetPosition(wp, _map.ViewportPointToLocation(leftTopPos));
                                 var routePoint = Route.Locations[1];
                                 routePoint.Latitude = info.TargetLat;
                                 routePoint.Longitude = info.TargetLng;
                                 routePoint.Altitude = info.TargetAlt;
                            }
                            break;
                    }
                };
            }

            public void RemoveCopter()
            {
                _map.Children.Remove(this.DotContainer);
                _map.Children.Remove(this.Track);
                _map.Children.Remove(this.Route);
                foreach (var item in this.Waypoints)
                {
                    _map.Children.Remove(item);
                }
            }

            public void RemoveMap_ViewChanged()
            {
                _map.ViewChangeOnFrame -= Map_ViewChanged;
            }

            public void SetShowroute(bool? showroute)
            {
             
                if (showroute ?? false )
                {
                    _map.Children.Add(this.Route);
                    MapLayer.SetZIndex(this.Route, 99);
                }
                   
                else
                    _map.Children.Remove(this.Route);
            }

            public void SetShowtrack(bool? showtrack)
            {
                if (showtrack ?? false)
                    _map.Children.Add(Track);
                else
                    _map.Children.Remove(Track);
            }




            public void SetEffect(bool selected)
            {
            
                if (selected)
                {
                    if (!DotContainer.Children.Contains(selectMarkup))
                        DotContainer.Children.Add(selectMarkup);
                }
                    
                else
                {
                    if (DotContainer.Children.Contains(selectMarkup))
                        DotContainer.Children.Remove(selectMarkup);
                }


                Waypoints.ForEach(wp => wp.Ismark = selected);


                /*
                Waypoints.ForEach(wp => wp.Effect = Dot.Effect);

                //Dot.Effect = selected ? _selectedEffect : null;
                Waypoints.ForEach(wp => wp.Effect = Dot.Effect);
                */
            }

            public void ResetRoute(int taskIndex)
            {
                var wpIndex = taskIndex - 1;    // Waypoints 中没有起飞点。
                Route.Locations.Clear();
                if (wpIndex >= 0 && wpIndex < Waypoints.Count)
                {
                    var info1 = _flightTaskManager.Tasks[wpIndex].SingleCopterInfos.Find(i => i.Copter == this.Copter);
                    var info2 = _flightTaskManager.Tasks[taskIndex].SingleCopterInfos.Find(i => i.Copter == this.Copter);
                    
                    Location loc1 = new Location(info1.TargetLat, info1.TargetLng);
                    Location loc2 = new Location(info2.TargetLat, info2.TargetLng);
                    Route.Locations.Add(loc1);
                    Route.Locations.Add(loc2);
                }
               
               
            }

            public void SetTaskEffect(int taskIndex)
            {
                var wpIndex = taskIndex - 1;    // Waypoints 中没有起飞点。
                if (wpIndex >= 0 && wpIndex < Waypoints.Count)
                {
                    Waypoints.ForEach(p => p.wp.Stroke = null);
                    var wp = Waypoints[wpIndex].wp;
                    wp.Stroke = _selectedTaskStroke;
                }
            }
			
			// 右击任务时触发
            public void SetTaskRightEffect(int taskIndex, bool flag)
            {
                var wpIndex = taskIndex - 1;    // Waypoints 中没有起飞点。
                if (wpIndex >= 0 && wpIndex < Waypoints.Count)
                {
                    // Waypoints.ForEach(p => p.Stroke = null);
                    var wp = Waypoints[wpIndex];
                    // Route.Locations.RemoveAt(wpIndex);
                    // Route.Visibility = Visibility.Hidden;
                    // wp.Stroke = _selectedTaskStroke;
                    // var flightTaskTmp = _flightTaskManager.Tasks[taskIndex];
                    if (!flag)
                    {
                        wp.Visibility = Visibility.Hidden;
                        //Route.Visibility = Visibility.Hidden;
                        //Route.Children.RemoveAt(1);
                        //Route.Locations.RemoveAt(taskIndex);
                    }
                    else
                    {
                        wp.Visibility = Visibility.Visible;
                        var info = _flightTaskManager.Tasks[taskIndex].SingleCopterInfos.Find(i => i.Copter == this.Copter);
                        //Route.Locations.Insert(taskIndex, new Location(info.TargetLat, info.TargetLng));
                        //Route.Visibility = Visibility.Visible;
                    }
                    
                }
            }
        }

        public class ShapesContainer : Grid
        {
            const double WAYPOINT_RADIUS = 6;
            public ShapesContainer(Brush brush)
            {
                wp = new Ellipse
                {
                    //Tag = WAYPOINT_TAG,
                    Fill = brush,
                    StrokeThickness = 2,
                    Width = WAYPOINT_RADIUS * 2,
                    Height = WAYPOINT_RADIUS * 2,
                    Margin = new Thickness(-WAYPOINT_RADIUS, -WAYPOINT_RADIUS, 0, 0)
                };
                this.Children.Add(wp);
                mark = new Rectangle();
                mark.Width = 6;
                mark.Height = 6;
                mark.HorizontalAlignment = HorizontalAlignment.Left;
                mark.VerticalAlignment = VerticalAlignment.Top;
                mark.Fill = new SolidColorBrush(Colors.Black);
                this.Children.Add(mark);

                mark.Visibility = Visibility.Hidden;
            }

            //public MapPolygon
            private bool isMarked;
            public Ellipse wp;
            public Rectangle mark;

            public bool Ismark
            {
                get
                {
                    return isMarked;
                }

                set
                {
                    isMarked = value;
                    mark.Visibility = value ? Visibility.Visible : Visibility.Hidden;
                }
            }
        }
    }
}
